import * as anchor from "@project-serum/anchor";
import { Program, Provider } from "@project-serum/anchor";
import { AccountInfo, MintInfo } from "@solana/spl-token";
import { Commitment, Keypair, PublicKey, TransactionInstruction } from "@solana/web3.js";
import { AnchorSdk, InstructionResult, TypedAccountParser } from "@strata-foundation/spl-utils";
import BN from "bn.js";
import { BondingHierarchy } from "./bondingHierarchy";
import { IPricingCurve } from "./curves";
import { CurveV0, ProgramStateV0, SplTokenBondingIDL, TokenBondingV0 } from "./generated/spl-token-bonding";
import { BondingPricing } from "./pricing";
export * from "./bondingHierarchy";
export * from "./curves";
export * from "./generated/spl-token-bonding";
export * from "./pricing";
export * from "./utils";
/**
 * The curve config required by the smart contract is unwieldy, implementors of `CurveConfig` wrap the interface
 */
interface ICurveConfig {
    toRawConfig(): CurveV0;
}
interface IPrimitiveCurve {
    toRawPrimitiveConfig(): any;
}
/**
 * Curve configuration for c(S^(pow/frac)) + b
 */
export declare class ExponentialCurveConfig implements ICurveConfig, IPrimitiveCurve {
    c: BN;
    b: BN;
    pow: number;
    frac: number;
    constructor({ c, b, pow, frac, }: {
        c?: number | BN;
        b?: number | BN;
        pow?: number;
        frac?: number;
    });
    toRawPrimitiveConfig(): any;
    toRawConfig(): CurveV0;
}
/**
 * Curve configuration that allows the curve to change parameters at discrete time offsets from the go live date
 */
export declare class TimeCurveConfig implements ICurveConfig {
    curves: {
        curve: IPrimitiveCurve;
        offset: BN;
    }[];
    addCurve(timeOffset: number, curve: IPrimitiveCurve): TimeCurveConfig;
    toRawConfig(): CurveV0;
}
export interface IInitializeCurveArgs {
    /** The configuration for the shape of curve */
    config: ICurveConfig;
    /** The payer to create this curve, defaults to provider.wallet */
    payer?: PublicKey;
}
export interface ICreateTokenBondingOutput {
    tokenBonding: PublicKey;
    targetMint: PublicKey;
    buyBaseRoyalties: PublicKey;
    buyTargetRoyalties: PublicKey;
    sellBaseRoyalties: PublicKey;
    sellTargetRoyalties: PublicKey;
    baseStorage: PublicKey;
}
export interface ICreateTokenBondingArgs {
    /** The payer to create this token bonding, defaults to provider.wallet */
    payer?: PublicKey;
    /** The shape of the bonding curve. Must be created using {@link SplTokenBonding.initializeCurve} */
    curve: PublicKey;
    /** The base mint that the `targetMint` will be priced in terms of. `baseMint` tokens will fill the bonding curve reserves */
    baseMint: PublicKey;
    /**
     * The mint this bonding curve will create on `buy`. If not provided, specify `targetMintDecimals` and it will create one for you
     *
     * It can be useful to pass the mint in if you're creating a bonding curve for an existing mint. Keep in mind,
     * the authority on this mint will need to be set to the token bonding pda
     */
    targetMint?: PublicKey;
    /**
     * **Default:** New generated keypair
     *
     * Pass in the keypair to use for the mint. Useful if you want a vanity keypair
     */
    targetMintKeypair?: anchor.web3.Keypair;
    /** If `targetMint` is not defined, will create a mint with this number of decimals */
    targetMintDecimals?: number;
    /**
     * Account to store royalties in terms of `baseMint` tokens when the {@link SplTokenBonding.buy} command is issued
     *
     * If not provided, will create an Associated Token Account with `buyBaseRoyaltiesOwner`
     */
    buyBaseRoyalties?: PublicKey;
    /** Only required when `buyBaseRoyalties` is undefined. The owner of the `buyBaseRoyalties` account. **Default:** `provider.wallet` */
    buyBaseRoyaltiesOwner?: PublicKey;
    /**
     * Account to store royalties in terms of `targetMint` tokens when the {@link SplTokenBonding.buy} command is issued
     *
     * If not provided, will create an Associated Token Account with `buyTargetRoyaltiesOwner`
     */
    buyTargetRoyalties?: PublicKey;
    /** Only required when `buyTargetRoyalties` is undefined. The owner of the `buyTargetRoyalties` account. **Default:** `provider.wallet` */
    buyTargetRoyaltiesOwner?: PublicKey;
    /**
     * Account to store royalties in terms of `baseMint` tokens when the {@link SplTokenBonding.sell} command is issued
     *
     * If not provided, will create an Associated Token Account with `sellBaseRoyaltiesOwner`
     */
    sellBaseRoyalties?: PublicKey;
    /** Only required when `sellBaseRoyalties` is undefined. The owner of the `sellBaseRoyalties` account. **Default:** `provider.wallet` */
    sellBaseRoyaltiesOwner?: PublicKey;
    /**
     * Account to store royalties in terms of `targetMint` tokens when the {@link SplTokenBonding.sell} command is issued
     *
     * If not provided, will create an Associated Token Account with `sellTargetRoyaltiesOwner`
     */
    sellTargetRoyalties?: PublicKey;
    /** Only required when `sellTargetRoyalties` is undefined. The owner of the `sellTargetRoyalties` account. **Default:** `provider.wallet` */
    sellTargetRoyaltiesOwner?: PublicKey;
    /**
     * General authority to change things like royalty percentages and freeze the curve. This is the least dangerous authority
     * **Default:** Wallet public key. Pass null to explicitly not set this authority.
     */
    generalAuthority?: PublicKey | null;
    /**
     * Authority to swap or change the reserve account. **This authority is dangerous. Use with care**
     *
     * From a trust perspective, this authority should almost always be held by another program that handles migrating bonding
     * curves, instead of by an individual.
     *
     * **Default:** null. You most likely don't need this permission, if it is being set you should do so explicitly.
     */
    reserveAuthority?: PublicKey | null;
    /**
     * Authority to swap or change the underlying curve. **This authority is dangerous. Use with care**
     *
     * From a trust perspective, this authority should almost always be held by another program that handles migrating bonding
     * curves, instead of by an individual.
     *
     * **Default:** null. You most likely don't need this permission, if it is being set you should do so explicitly.
     */
    curveAuthority?: PublicKey | null;
    /**
     * The reserves of the bonding curve. When {@link SplTokenBonding.buy} is called, `baseMint` tokens are stored here.
     * When {@link SplTokenBonding.sell} is called, `baseMint` tokens are returned to the callee from this account
     *
     * Optionally, this account can have an authority _not_ owned by the spl-token-bonding program. In this case, a bonding curve
     * is created with {@link SplTokenBonding.sell} disabled. This allows the bonding curve contract to be used like a
     * marketplace to sell a new token
     *
     * **Default:** creates this account for you, owned by the token bonding program
     */
    baseStorage?: PublicKey;
    /** Number from 0 to 100 */
    buyBaseRoyaltyPercentage: number;
    /** Number from 0 to 100 */
    buyTargetRoyaltyPercentage: number;
    /** Number from 0 to 100 */
    sellBaseRoyaltyPercentage: number;
    /** Number from 0 to 100 */
    sellTargetRoyaltyPercentage: number;
    /** Maximum `targetMint` tokens this bonding curve will mint before disabling {@link SplTokenBonding.buy}. **Default:** infinite */
    mintCap?: BN;
    /** Maximum `targetMint` tokens that can be purchased in a single call to {@link SplTokenBonding.buy}. Useful for limiting volume. **Default:** 0 */
    purchaseCap?: BN;
    /** The date this bonding curve will go live. Before this date, {@link SplTokenBonding.buy} and {@link SplTokenBonding.sell} are disabled. **Default:** 1 second ago */
    goLiveDate?: Date;
    /** The date this bonding curve will shut down. After this date, {@link SplTokenBonding.buy} and {@link SplTokenBonding.sell} are disabled. **Default:** null */
    freezeBuyDate?: Date;
    /** Should this bonding curve be frozen initially? It can be unfrozen using {@link SplTokenBonding.updateTokenBonding}. **Default:** false */
    buyFrozen?: boolean;
    /**
     * Multiple bonding curves can exist for a given target mint.
     * 0 is reserved for the one where the program owns mint authority and can mint new tokens. All other curves may exist as
     * markeplace curves
     */
    index?: number;
}
export interface IUpdateTokenBondingArgs {
    /** The bonding curve to update */
    tokenBonding: PublicKey;
    /** Number from 0 to 100. **Default:** current */
    buyBaseRoyaltyPercentage?: number;
    /** Number from 0 to 100. **Default:** current */
    buyTargetRoyaltyPercentage?: number;
    /** Number from 0 to 100. **Default:** current */
    sellBaseRoyaltyPercentage?: number;
    /** Number from 0 to 100. **Default:** current */
    sellTargetRoyaltyPercentage?: number;
    /** A new account to store royalties. **Default:** current */
    buyBaseRoyalties?: PublicKey;
    /** A new account to store royalties. **Default:** current */
    buyTargetRoyalties?: PublicKey;
    /** A new account to store royalties. **Default:** current */
    sellBaseRoyalties?: PublicKey;
    /** A new account to store royalties. **Default:** current */
    sellTargetRoyalties?: PublicKey;
    generalAuthority?: PublicKey | null;
    /** Should this bonding curve be frozen, disabling buy and sell? It can be unfrozen using {@link SplTokenBonding.updateTokenBonding}. **Default:** current */
    buyFrozen?: boolean;
}
export interface IBuyArgs {
    tokenBonding: PublicKey;
    /** The payer to run this transaction, defaults to provider.wallet */
    payer?: PublicKey;
    /** The source account to purchase with. **Default:** ata of `sourceAuthority` */
    source?: PublicKey;
    /** The source destination to purchase to. **Default:** ata of `sourceAuthority` */
    destination?: PublicKey;
    /** The wallet funding the purchase. **Default:** Provider wallet */
    sourceAuthority?: PublicKey;
    /** Must provide either base amount or desired target amount */
    desiredTargetAmount?: BN | number;
    baseAmount?: BN | number;
    /** Decimal number. max price will be (1 + slippage) * price_for_desired_target_amount */
    slippage: number;
}
export interface ISwapArgs {
    baseMint: PublicKey;
    targetMint: PublicKey;
    /** The payer to run this transaction, defaults to provider.wallet */
    payer?: PublicKey;
    /** The wallet funding the purchase. **Default:** Provider wallet */
    sourceAuthority?: PublicKey;
    /** The amount of baseMint to purchase with */
    baseAmount: BN | number;
    /** The slippage PER TRANSACTION */
    slippage: number;
    /** Optionally inject extra instructions before each trade. Usefull for adding txn fees */
    extraInstructions?: (args: {
        tokenBonding: ITokenBonding;
        isBuy: boolean;
        amount: BN;
    }) => Promise<InstructionResult<null>>;
}
export interface ISellArgs {
    tokenBonding: PublicKey;
    /** The payer to run this transaction, defaults to provider.wallet */
    payer?: PublicKey;
    source?: PublicKey /** `targetMint` source account to sell from. **Default:** ATA of sourceAuthority */;
    destination?: PublicKey /** `baseMint` destination for tokens from the reserve. **Default:** ATA of wallet */;
    sourceAuthority?: PublicKey /** **Default:** wallet */;
    targetAmount: BN | number /** The amount of `targetMint` tokens to sell. */;
    slippage: number;
}
export interface IBuyBondingWrappedSolArgs {
    amount: BN | number /** The amount of wSOL to buy. If a number, multiplied out to get lamports. If BN, it's lamports */;
    destination?: PublicKey /** The destination twSOL account. **Default:** ATA of owner */;
    source?: PublicKey /** The source of non-wrapped SOL */;
    payer?: PublicKey;
}
export interface ISellBondingWrappedSolArgs {
    amount: BN | number /** The amount of wSOL to buy. If a number, multiplied out to get lamports. If BN, it's lamports */;
    source?: PublicKey /** The twSOL source account. **Default:** ATA of owner */;
    destination?: PublicKey /** The destination to send the actual SOL lamports. **Default:** provider wallet */;
    owner?: PublicKey /** The owner of the twSOL source account. **Default:** provider wallet */;
    payer?: PublicKey;
    all?: boolean /** Sell all and close this account? **Default:** false */;
}
/**
 * Unified token bonding interface wrapping the raw TokenBondingV0
 */
export interface ITokenBonding extends TokenBondingV0 {
    publicKey: PublicKey;
}
/**
 * Unified curve interface wrapping the raw CurveV0
 */
export interface ICurve extends CurveV0 {
    publicKey: PublicKey;
}
export declare class SplTokenBonding extends AnchorSdk<SplTokenBondingIDL> {
    state: ProgramStateV0 | undefined;
    static ID: anchor.web3.PublicKey;
    static WRAPPED_SOL_MINT: anchor.web3.PublicKey;
    static init(provider: Provider, splTokenBondingProgramId?: PublicKey): Promise<SplTokenBonding>;
    constructor(provider: Provider, program: Program<SplTokenBondingIDL>);
    curveDecoder: TypedAccountParser<ICurve>;
    tokenBondingDecoder: TypedAccountParser<ITokenBonding>;
    getTokenBonding(tokenBondingKey: PublicKey): Promise<ITokenBonding | null>;
    getCurve(curveKey: PublicKey): Promise<ICurve | null>;
    /**
     * This is an admin function run once to initialize the smart contract.
     *
     * @returns Instructions needed to create sol storage
     */
    initializeSolStorageInstructions(): Promise<InstructionResult<null>>;
    /**
     * Admin command run once to initialize the smart contract
     */
    initializeSolStorage(): Promise<null>;
    /**
     * Create a curve shape for use in a TokenBonding instance
     *
     * @param param0
     * @returns
     */
    initializeCurveInstructions({ payer, config: curveConfig, }: IInitializeCurveArgs): Promise<InstructionResult<{
        curve: PublicKey;
    }>>;
    /**
     * See {@link initializeCurve}
     * @param args
     * @returns
     */
    initializeCurve(args: IInitializeCurveArgs): Promise<PublicKey>;
    /**
     * Get the PDA key of a TokenBonding given the target mint and index
     *
     * `index` = 0 is the default bonding curve that can mint `targetMint`. All other curves are curves that allow burning of `targetMint` for some different base.
     *
     * @param targetMint
     * @param index
     * @returns
     */
    static tokenBondingKey(targetMint: PublicKey, index?: number, programId?: PublicKey): Promise<[PublicKey, number]>;
    /**
     * Create a bonding curve
     *
     * @param param0
     * @returns
     */
    createTokenBondingInstructions({ generalAuthority, curveAuthority, reserveAuthority, payer, curve, baseMint, targetMint, baseStorage, buyBaseRoyalties, buyBaseRoyaltiesOwner, buyTargetRoyalties, buyTargetRoyaltiesOwner, sellBaseRoyalties, sellBaseRoyaltiesOwner, sellTargetRoyalties, sellTargetRoyaltiesOwner, buyBaseRoyaltyPercentage, buyTargetRoyaltyPercentage, sellBaseRoyaltyPercentage, sellTargetRoyaltyPercentage, mintCap, purchaseCap, goLiveDate, // 10 secs ago
    freezeBuyDate, targetMintDecimals, targetMintKeypair, buyFrozen, index, }: ICreateTokenBondingArgs): Promise<InstructionResult<ICreateTokenBondingOutput>>;
    /**
     * General utility function to check if an account exists
     * @param account
     * @returns
     */
    accountExists(account: anchor.web3.PublicKey): Promise<boolean>;
    /**
     * Runs {@link `createTokenBondingInstructions`}
     *
     * @param args
     * @returns
     */
    createTokenBonding(args: ICreateTokenBondingArgs): Promise<ICreateTokenBondingOutput>;
    /**
     * Update a bonding curve.
     *
     * @param param0
     * @returns
     */
    updateTokenBondingInstructions({ tokenBonding, buyBaseRoyaltyPercentage, buyTargetRoyaltyPercentage, sellBaseRoyaltyPercentage, sellTargetRoyaltyPercentage, buyBaseRoyalties, buyTargetRoyalties, sellBaseRoyalties, sellTargetRoyalties, generalAuthority, buyFrozen, }: IUpdateTokenBondingArgs): Promise<InstructionResult<null>>;
    /**
     * Runs {@link updateTokenBonding}
     * @param args
     */
    updateTokenBonding(args: IUpdateTokenBondingArgs): Promise<void>;
    /**
     * Instructions to buy twSOL from normal SOL.
     *
     * We wrap SOL so that the bonding contract isn't soaking up a bunch o SOL and damaging the security of the network.
     * The plan is to create a DAO for Strata that will govern what happens with this SOL.
     *
     * @param param0
     * @returns
     */
    buyBondingWrappedSolInstructions({ payer, destination, source, amount, }: IBuyBondingWrappedSolArgs): Promise<InstructionResult<{
        destination: PublicKey;
    }>>;
    /**
     * Invoke `buyBondingWrappedSol` instructions
     * @param args
     * @returns
     */
    buyBondingWrappedSol(args: IBuyBondingWrappedSolArgs): Promise<{
        destination: PublicKey;
    }>;
    /**
     * Instructions to sell twSOL back into normal SOL.
     *
     * @param param0
     * @returns
     */
    sellBondingWrappedSolInstructions({ source, owner, destination, amount, all, }: ISellBondingWrappedSolArgs): Promise<InstructionResult<null>>;
    /**
     * Execute `sellBondingWrappedSolInstructions`
     * @param args
     * @returns
     */
    sellBondingWrappedSol(args: ISellBondingWrappedSolArgs): Promise<null>;
    /**
     * Create a temporary account with `amount` twSOL, the token bonding wrapped sol mint.
     *
     * @param param0
     * @returns
     */
    createTemporaryWSolAccount({ payer, owner, amount, }: {
        owner: PublicKey;
        payer: PublicKey;
        amount: number;
    }): Promise<{
        signer: Keypair;
        firstInstructions: TransactionInstruction[];
        lastInstructions: TransactionInstruction[];
    }>;
    /**
     * Issue a command to buy `targetMint` tokens with `baseMint` tokens.
     *
     * @param param0
     * @returns
     */
    buyInstructions({ tokenBonding, source, sourceAuthority, destination, desiredTargetAmount, baseAmount, slippage, payer, }: IBuyArgs): Promise<InstructionResult<null>>;
    /**
     * Runs {@link buy}
     * @param args
     */
    buy(args: IBuyArgs): Promise<void>;
    getTokenAccountBalance(account: PublicKey, commitment?: Commitment): Promise<BN>;
    /**
     * Swap from any base mint to any target mint that are both on a shared link of bonding curves.
     * Intelligently traverses using either buy or sell, executing multiple txns to either sell baseAmount
     * or buy with baseAmount
     *
     * @param param0
     */
    swap({ payer, sourceAuthority, baseMint, targetMint, baseAmount, slippage, extraInstructions, }: ISwapArgs): Promise<{
        targetAmount: number;
    }>;
    getState(): Promise<ProgramStateV0 | null>;
    /**
     * Instructions to burn `targetMint` tokens in exchange for `baseMint` tokens
     *
     * @param param0
     * @returns
     */
    sellInstructions({ tokenBonding, source, sourceAuthority, destination, targetAmount, slippage, payer, }: ISellArgs): Promise<InstructionResult<null>>;
    /**
     * Runs {@link sell}
     * @param args
     */
    sell(args: ISellArgs): Promise<void>;
    /**
     * Get a class capable of displaying pricing information or this token bonding at its current reserve and supply
     *
     * @param tokenBonding
     * @returns
     */
    getBondingPricingCurve(tokenBonding: PublicKey): Promise<IPricingCurve>;
    /**
     * Given some reserves and supply, get a pricing model for a curve at `key`.
     *
     * @param key
     * @param baseStorage
     * @param baseMint
     * @param targetMint
     * @returns
     */
    getPricingCurve(key: PublicKey, baseStorage: AccountInfo, baseMint: MintInfo, targetMint: MintInfo): Promise<IPricingCurve>;
    getPricing(tokenBondingKey: PublicKey | undefined): Promise<BondingPricing | undefined>;
    /**
     * Fetch the token bonding curve and all of its direct ancestors
     *
     * @param tokenBondingKey
     * @returns
     */
    getBondingHierarchy(tokenBondingKey: PublicKey | undefined, stopAtMint?: PublicKey | undefined): Promise<BondingHierarchy | undefined>;
}
//# sourceMappingURL=index.d.ts.map