"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BondingPricing = void 0;
const spl_token_1 = require("@solana/spl-token");
const _1 = require(".");
/**
 * Traverse a bonding hierarchy, executing func and accumulating
 * the results until destination token
 *
 * @param param0
 * @returns
 */
function reduce({ hierarchy, func, initial, destination, }) {
    if (!hierarchy) {
        return initial;
    }
    if (destination === null || destination === void 0 ? void 0 : destination.equals(spl_token_1.NATIVE_MINT)) {
        destination = _1.SplTokenBonding.WRAPPED_SOL_MINT;
    }
    let current = hierarchy;
    let value = func(initial, current);
    while (!current.tokenBonding.baseMint.equals(destination)) {
        current = current.parent;
        if (!current) {
            throw new Error(`Base mint ${destination.toBase58()} is not in the hierarchy for ${hierarchy.tokenBonding.publicKey.toBase58()}`);
        }
        value = func(value, current);
    }
    return value;
}
/**
 * Traverse a bonding hierarchy, executing func and accumulating
 * the results until destination token starting from parent going to children
 *
 * @param param0
 * @returns
 */
function reduceFromParent({ hierarchy, func, initial, destination, }) {
    if (!hierarchy) {
        return initial;
    }
    if (destination === null || destination === void 0 ? void 0 : destination.equals(spl_token_1.NATIVE_MINT)) {
        destination = _1.SplTokenBonding.WRAPPED_SOL_MINT;
    }
    let current = hierarchy;
    while (!current.tokenBonding.baseMint.equals(destination)) {
        current = current.parent;
        if (!current) {
            throw new Error(`Base mint ${destination.toBase58()} is not in the hierarchy for ${hierarchy.tokenBonding.publicKey.toBase58()}`);
        }
    }
    destination = hierarchy.tokenBonding.targetMint;
    let value = func(initial, current);
    while (!current.tokenBonding.targetMint.equals(destination)) {
        current = current.child;
        value = func(value, current);
    }
    return value;
}
class BondingPricing {
    constructor(args) {
        this.hierarchy = args.hierarchy;
    }
    current(baseMint = this.hierarchy.tokenBonding.baseMint) {
        return reduce({
            hierarchy: this.hierarchy,
            func: (acc, current) => {
                return acc * current.pricingCurve.current();
            },
            initial: 1,
            destination: baseMint,
        });
    }
    locked(baseMint = this.hierarchy.tokenBonding.baseMint) {
        return reduce({
            hierarchy: this.hierarchy.parent,
            func: (acc, current) => {
                return acc * current.pricingCurve.current();
            },
            initial: this.hierarchy.pricingCurve.locked(),
            destination: baseMint,
        });
    }
    swap(baseAmount, baseMint, targetMint) {
        const lowMint = this.hierarchy.lowest(baseMint, targetMint);
        const highMint = lowMint.equals(baseMint) ? targetMint : baseMint;
        const isBuying = lowMint.equals(targetMint);
        const path = this.hierarchy.path(lowMint, highMint);
        if (path.length == 0) {
            throw new Error(`No path from ${baseMint} to ${targetMint}`);
        }
        if (isBuying) {
            return path.reverse().reduce((amount, { pricingCurve, tokenBonding }) => {
                return pricingCurve.buyWithBaseAmount(amount, tokenBonding.buyBaseRoyaltyPercentage, tokenBonding.buyTargetRoyaltyPercentage);
            }, baseAmount);
        }
        else {
            return path.reduce((amount, { pricingCurve, tokenBonding }) => {
                return pricingCurve.sellTargetAmount(amount, tokenBonding.sellBaseRoyaltyPercentage, tokenBonding.sellTargetRoyaltyPercentage);
            }, baseAmount);
        }
    }
    sellTargetAmount(targetAmountNum, baseMint = this.hierarchy.tokenBonding.baseMint) {
        return reduce({
            hierarchy: this.hierarchy,
            func: (acc, current) => {
                return current.pricingCurve.sellTargetAmount(acc, current.tokenBonding.sellBaseRoyaltyPercentage, current.tokenBonding.sellTargetRoyaltyPercentage);
            },
            initial: targetAmountNum,
            destination: baseMint,
        });
    }
    buyTargetAmount(targetAmountNum, baseMint = this.hierarchy.tokenBonding.baseMint) {
        return reduce({
            hierarchy: this.hierarchy,
            func: (acc, current) => {
                return current.pricingCurve.buyTargetAmount(acc, current.tokenBonding.buyBaseRoyaltyPercentage, current.tokenBonding.buyTargetRoyaltyPercentage);
            },
            initial: targetAmountNum,
            destination: baseMint,
        });
    }
    buyWithBaseAmount(baseAmountNum, baseMint = this.hierarchy.tokenBonding.baseMint) {
        return reduceFromParent({
            hierarchy: this.hierarchy,
            func: (acc, current) => {
                return current.pricingCurve.buyWithBaseAmount(acc, current.tokenBonding.buyBaseRoyaltyPercentage, current.tokenBonding.buyTargetRoyaltyPercentage);
            },
            initial: baseAmountNum,
            destination: baseMint,
        });
    }
}
exports.BondingPricing = BondingPricing;
//# sourceMappingURL=pricing.js.map